#!/usr/bin/env python3
"""
  Course:  Data Structures and Algorithms for CL III - WS2425
  Assignment:  7
  Author(s):
  Description:
  Honor Code:  I/We pledge that this program represents my/our own work.
"""

import csv
import numpy as np

# Definitions of IPA vowels for convenience (only non-accented versions).
# Can be incomplete, but "good enough" for proof of concept.
IPA_VOWELS = {'ɐ', 'ɑ', 'ɒ', 'ɔ', 'ɘ', 'ə', 'ɛ', 'ɜ', 'ɞ', 'ɤ', 'ɨ',
              'ɪ', 'ɯ', 'ɵ', 'ɶ', 'ʉ', 'ʊ', 'ʌ', 'ʏ', 'i', 'y', 'e',
              'ø', 'œ', 'æ', 'a', 'u', 'o'}


def is_vowel(char):
    return char in IPA_VOWELS


def is_cons(char):
    return not is_vowel(char)


def edit_distance(s1, s2, weights=[0.5, 1.0, 2.0, 0.5, 2.0]):
    """ Find the weighted edit distance between two IPA strings.

    You are not allowed to use any external library / function for
    implementing the minimum edit distance algorithm. You should
    implement the edit distance (similar to the dynamic programming
    example introduced in the class) yourself.

    You can use is_vowel() above for determining if a symbol is a
    vowel. Any character which is not a vowel is assumed to be a
    consonant.

    In case there are more than one alignment with the minimum cost,
    pick one of them arbitrarily.

    Parameters
    ----------
    s1, s2: sequences to be aligned
    weights: a sequence of length 5, corresponding to
        - penalty of a vowel-vowel replacement
        - penalty of a consonant-consonant replacement
        - penalty of a vowel-consonant replacement
        - penalty if a vowel deletion or insertion
        - penalty of a consonant deletion or insertion

    Return value
    ------------
    The minimum edit distance based on the given weights (a float).
    """
    pass


def language_distance(l1, l2, lexfile='northeuralex-0.9-forms.tsv',
                      weights=[0.5, 1.0, 2.0, 0.5, 2.0]):
    """
    Determine the similarity of two languages in the lex file based on edit distance.

    We use the edit distance implementation to find a distance measure
    between two languages based on NorthEuralex.

    We define the similarity as average normalized edit distance of all concepts.
    That is, we take the normalized edit distance, which is the edit
    distance divided to the maximum possible edit distance between two
    strings. For simplicity, we define it as the sum of the highest
    cost replacements for all segments in the shorter string, followed
    by as many highest cost insert/delete operations as the length
    difference between the strings.

    For example, with the default weights provided, the maximum cost between a sequence of length 5 and
    another sequence of length 8 should be calculated as:
    5 * 2.0 + 3 * 2.0 where the first 2.0 is the replacement cost for
    a vowel and a consonant, and second 2.0 is the insertion cost for
    a consonant, which happens to be the same here, but can be
    different if a different set of weights are specified.

    Calculate the edit distance only if both languages have a valid
    pronunciation listed in NorthEuralex for a concept. If no
    pronunciation exists for one of the languages, skip the concept.
    If multiple pronunciations exist for one or both languages for a
    particular concept, take the minimum edit distance among all pairs
    of pronunciations.

    Parameters
    ----------
    l1: The language code of the first language (as in Language_ID
           column of NorthEuralex data file).
    l2: The language code of the second language.
    lexfile: File name for the NorthEuralex lexical data.
    weights: The weights used for edit distance calculation (see
             the description of weights in documentation of
             edit_distance()).

    Return value
    ------------
    The aggregate edit distance for all common concepts as defined above.

    """
    pass


if __name__ == '__main__':
    for l2 in ('nld', 'dan', 'nor', 'eng', 'fra', 'ita', 'pol', 'rus',
               'fin', 'bul'):
        print(f"deu-{l2}:", language_distance("deu", l2))
